@@ -11,26 +11,25 @@
 #include <QList>
 #include <QMap>
 #include <QIODevice>
-#include "fileinfo.h"
 #include "qmmp.h"
 #include "audioparameters.h"
 #include "channelmap.h"
-#include "decoderfactory.h"
 
 class QmmpPluginCache;
+class DecoderFactory;
 
 /*! @brief The Decoder class provides the base interface class of audio decoders.
  * @author Brad Hughes <bhughes@trolltech.com>
  * @author Ilya Kotov <forkotov02@ya.ru>
  */
-class Decoder
+class QMMP_EXPORT Decoder
 {
 public:
     /*!
      * Object contsructor.
      * @param input QIODevice-based input source.
      */
-    Decoder(QIODevice *input = 0);
+    Decoder(QIODevice *input = nullptr);
     /*!
      * Destructor.
      */
@@ -103,17 +102,32 @@ public:
     bool hasMetaData() const;
     /*!
      * Takes metadata out of decoder and returns it.
-     * Attention: hasMetaData() should return \b true before use of this fuction.
+     * Attention: hasMetaData() should return \b true before using of this fuction.
      */
     QMap<Qmmp::MetaData, QString> takeMetaData();
     /*!
-     * Returns DecoderFactory pointer which supports file \b path or 0 if file \b path is unsupported
+     * Sets extra track property.
+     * \param key Property key.
+     * \param value Property value.
+     */
+    void setProperty(Qmmp::TrackProperty key, const QVariant &value);
+    /*!
+     * Updates all extra track properties.
+     * \param properties New track properties.
+     */
+    void setProperties(const QMap<Qmmp::TrackProperty, QString> &properties);
+    /*!
+     * Returns track properties
+     */
+    const QMap<Qmmp::TrackProperty, QString> &properties() const;
+    /*!
+     * Returns DecoderFactory pointer which supports file \b path or nullptr if file \b path is unsupported
      * @param path Full local file path.
      * @param useContent Content-based file type determination (\b true - enabled, \b false - disabled)
      */
     static DecoderFactory *findByFilePath(const QString &path, bool useContent = false);
     /*!
-     * Returns DecoderFactory pointer which supports mime type \b mime or \b 0 if mime type \b mime is unsupported
+     * Returns DecoderFactory pointer which supports mime type \b mime or \b nullptr if mime type \b mime is unsupported
      */
     static DecoderFactory *findByMime(const QString &mime);
     /*!
@@ -122,7 +136,7 @@ public:
      */
     static DecoderFactory *findByContent(QIODevice *input);
     /*!
-     * Returns DecoderFactory pointer which supports protocol \b p or \b 0 if \b url is not supported.
+     * Returns DecoderFactory pointer which supports protocol \b p or \b nullptr if \b protocol is not supported.
      */
     static DecoderFactory *findByProtocol(const QString &p);
     /*!
@@ -138,11 +152,15 @@ public:
      * Returns a list of enabled decoder factories.
      */
     static QList<DecoderFactory*> enabledFactories();
+    /*!
+     * Returns a list of file name filters, i.e. "*.mp3 *.mpg".
+     */
+    static QStringList nameFilters();
     /*!
      * Returns plugin file path.
      * @param factory Decoder plugin factory.
      */
-    static QString file(DecoderFactory *factory);
+    static QString file(const DecoderFactory *factory);
     /*!
      * Returns a list of supported protocols (including meta-protocols).
      * This fuction ignores disabled decoders.
@@ -153,12 +171,12 @@ public:
      * @param factory Decoder plugin factory.
      * @param enable Plugin enable state (\b true - enable, \b false - disable)
      */
-    static void setEnabled(DecoderFactory* factory, bool enable = true);
+    static void setEnabled(DecoderFactory *factory, bool enable = true);
     /*!
      * Returns \b true if input plugin is enabled, otherwise returns \b false
      * @param factory Decoder plugin factory.
      */
-    static bool isEnabled(DecoderFactory* factory);
+    static bool isEnabled(const DecoderFactory* factory);
 
 protected:
     /*!
@@ -186,6 +204,7 @@ private:
     static QList<QmmpPluginCache*> *m_cache;
     static QStringList m_disabledNames;
     AudioParameters m_parameters;
+    QMap<Qmmp::TrackProperty, QString> m_properties;
     QIODevice *m_input;
     bool m_hasMetaData;
     QMap<Qmmp::MetaData, QString> m_metaData;
